<?php

function _drd_save_servers($servers) {
  variable_set('drd_servers', $servers);
}

function _drd_load_servers() {
  static $servers;
  if (!isset($servers)) {
    $servers = variable_get('drd_servers', array());
    foreach ($servers as $key => $server) {
      $server += array(
        'weight' => 0,
        'disabled' => FALSE,
        'url' => '',
        'user' => '',
        'password' => '',
        'svn' => array(
          'enabled' => FALSE,
          'user' => '',
          'password' => '',
          'force_external' => FALSE,
        ),
        'disabled_domains' => '',

      );
      $servers[$key] = $server;
    }
  }
  return $servers;
}

function _drd_server_sort($a, $b) {
  if ($a['disabled'] !== $b['disabled']) {
    return $a['disabled'] ? 1 : 0;
  }
  return ($a['weight'] > $b['weight']);
}

function drd_dashboard() {
  drupal_set_title('Drupal Remote Dashboard');
  $servers = _drd_load_servers();
  uasort($servers, '_drd_server_sort');
  $domain_output = '';
  $server_output = '';
  $output  = '<div id="drd-dashboard">';
  $output .= '  <div class="drd-left-pane">';
  $output .= '    <div id="server-toolbars">';
  $output .= '      <div class="toolbar general-toolbar">'. t('Toolbar:') . _drd_general_toolbar() .'</div>';
  $output .= '      <div class="toolbar all-server-toolbar">'. t('Update all servers:') . _drd_server_toolbar(FALSE, FALSE, TRUE, TRUE) .'</div>';
  $output .= '      <div class="toolbar all-server-all-domains-toolbar">'. t('Update all domains of all servers:') . _drd_domain_toolbar() .'</div>';
  $output .= '    </div>';
  $output .= '    <div id="servers">';
  foreach ($servers as $id => $server) {
    if (isset($server['deleted']) && $server['deleted']) {
      continue;
    }
    list($part_main, $part_domain, $part_server) = _drd_get_server_markup($id, $server['url'], $server['disabled'], $server['svn']['enabled']);
    $output .= $part_main;
    $domain_output .= $part_domain;
    $server_output .= $part_server;
  }
  $form = drupal_get_form('drd_dashboard_edit_server_form');
  $output .= '</div>';
  $output .= '<div id="error-messages" class="element-hidden">';
  $output .= '<div class="title">'. t('Error messages') .'</div>'. _drd_icon('delete', 'trash', t('Delete messages'));
  $output .= '<div class="content">&nbsp;</div>';
  $output .= '</div>';
  $output .= '</div>';
  $output .= '<div id="server-domain-details">';
  $output .= $domain_output;
  $output .= '</div>';
  $output .= '<div id="server-details">';
  $output .= $server_output;
  $output .= '</div>';
  $output .= '<div id="server-edit" class="element-hidden">'. drupal_render($form) .'</div>';
  $output .= '<div class="clearfix">&nbsp;</div>';
  $output .= '</div>';
  return $output;
}

function _drd_get_server_markup($id, $url, $disabled, $svn_enabled) {
  $main_part    = '<div id="id'. $id .'" class="server">';
  $main_part   .= '  <div class="server-name">'. t('Unknown') .'</div>';
  $main_part   .=    _drd_icon_alerts('server');
  $main_part   .= ($disabled) ? '' : '  <div class="server-show-detail ui-icon ui-icon-triangle-1-e">'. t('Show Detail') .'</div>';
  $main_part   .= '  <div class="server-url"><div class="plain-url element-hidden">'. $url .'</div><div class="url">'. l($url, 'http://'. $url, array('external' => TRUE, 'attributes' => array('target' => '_blank'))) .'</div></div>';
  $main_part   .= '  <div class="toolbar server-toolbar">';
  $main_part   .=      _drd_server_toolbar(TRUE, !$disabled, !$disabled, (!$disabled && $svn_enabled));
  $main_part   .= '    <div class="last-updated">&nbsp;</div>';
  $main_part   .= '  </div>';
  $main_part   .= '</div>';
  if ($disabled) {
    $domain_part = '';
    $server_part = '';
  }
  else {
    $domain_part  = '<div id="id'. $id .'" class="server">';
    $domain_part .= ' <div class="server-domain-detail element-hidden">';
    $domain_part .= '   <div class="toolbar all-domains-toolbar">'. t('Update all domains:') . _drd_domain_toolbar() .'</div>';
    $domain_part .= '   <div class="server-domains">&nbsp;</div>';
    $domain_part .= ' </div>';
    $domain_part .= '</div>';
    $server_part  = '<div id="id'. $id .'" class="server">';
    $server_part .= ' <div class="server-detail">';
    $server_part .=     _drd_info_box('server', 'info', t('Status Report ('. $url .')'));
    $server_part .=     _drd_info_box('server', 'message-list', t('Messages ('. $url .')'));
    $server_part .=     _drd_info_box('server', 'svn-status', t('SVN Status ('. $url .')'));
    $server_part .=     _drd_info_box('server', 'svn-update', t('SVN Update Report ('. $url .')'));
    $server_part .=     _drd_info_box('server', 'php-error-log', t('PHP Error Log ('. $url .')'));
    $server_part .= ' </div>';
    $server_part .= '</div>';
  }
  return array($main_part, $domain_part, $server_part);
}

function _drd_icon($class, $icon, $title) {
  return '<div class="'. $class .' icon-inline ui-icon ui-icon-'. $icon .'" title="'. $title .'">'. $title .'</div>';
}

function _drd_icon_refresh($class) {
  return _drd_icon($class, 'refresh refresh', t('Refresh'));
}

function _drd_icon_alerts($mode) {
  return '<div class="'. $mode .'-alert '. $mode .'-alert-error"><div class="toggle-alert element-hidden">'.
    _drd_icon('drd-error', 'alert', t('Error')) .'</div></div>'.
    '<div class="'. $mode .'-alert '. $mode .'-alert-warning"><div class="toggle-alert element-hidden">'.
    _drd_icon('drd-warning', 'alert', t('Warning')) .'</div></div>';
}

function _drd_general_toolbar() {
  return _drd_icon('new', 'document-b', t('Add new server')) .
         _drd_icon('flush-cache', 'heart', t('Flush DRD Cache'));
}

function _drd_info_box($mode, $action, $title) {
  $close_button = ($mode == 'server') ? _drd_icon('close', 'close', t('Close')) : '';
  $refresh_button = ($action == 'message-list') ? _drd_icon('delete', 'trash', t('Delete messages')) : _drd_icon_refresh($action);
  $refresh_button .= ($action == 'list-updates') ? _drd_icon('toggle-update-list', 'pin-w', t('Toggle all/outdated')) : '';
  return '<div class="'. $mode .'-info-box '. $mode .'-'. $action .'">'.
    '<div class="title">'. $title .'</div>'.
    $refresh_button .
    $close_button .
    '<div class="last-updated">&nbsp;</div>'.
    '<div class="content">&nbsp;</div>'.
    '</div>';
}

function _drd_server_toolbar($include_edit, $include_info, $include_domains, $include_svn) {
  $edit = $include_edit ? _drd_icon('edit', 'pencil', t('Edit')) . _drd_icon('delete', 'trash', t('Delete')) : '';
  $svn = ($include_edit && $include_svn) ? '<div class="server-svn-revision">?</div>' : '';
  $domains_icon = ($include_domains) ? _drd_icon('domains', 'home', t('Domains')) : '';
  $svn_icons = ($include_svn) ? _drd_icon('svn-status', 'shuffle', t('SVN Status')) . _drd_icon('svn-update', 'transfer-e-w', t('SVN Update')) : '';
  if ($include_info) {
    $svn_info = ($include_svn) ? _drd_icon('server-svn-status', 'shuffle', t('Show SVN Status Info')) . _drd_icon('server-svn-update', 'transfer-e-w', t('Show SVN Update Info')) : '';
    $svn = '<div class="info-toolbar">'. t('Info:') .
      _drd_icon('server-info', 'info', t('Show Status Report')) .
      _drd_icon('server-message-list', 'mail-closed', t('Show Messages')) .
      _drd_icon('server-php-error-log', 'mail-open', t('Show server PHP Error Log')) .
      $svn_info .
      '</div>'.
      $svn;
  }
  return $edit .
    _drd_icon('info', 'info', t('Get server info')) .
    $domains_icon .
    _drd_icon('php-error-log', 'mail-open', t('Get server PHP Error Log')) .
    $svn_icons .
    $svn;
}

function _drd_domain_toolbar() {
  return ''.
    _drd_icon('info', 'info', t('Info')) .
    _drd_icon('flush-cache', 'heart', t('Flush Cache')) .
    _drd_icon('run-cron', 'calendar', t('Run Cron')) .
    _drd_icon('list-updates', 'flag', t('List Available Updates')) .
    _drd_icon('run-update', 'wrench', t('Run Update')) .
    _drd_icon('switch-maintenance', 'cancel', t('Switch Maintenance-mode')) .
    _drd_icon('update-translation', 'contact', t('Update Translation'));
}

function _drd_get_url_key($id, $domain_id = NULL, $domain_url = NULL) {
  $servers = _drd_load_servers();
  if (isset($domain_id) && ($domain_url != $servers[$id]['url'])) {
    $plain_url = $domain_url;
    $key = 'drd_domain_session_'. $id .'_'. $domain_id;
  }
  else {
    $plain_url = $servers[$id]['url'];
    $key = 'drd_server_session_'. $id;
  }
  $cache = cache_get($key, 'cache_drd');
  if ($cache) {
    list($plain_url, $session) = $cache->data;
  }
  else {
    list($plain_url, $session) = array($plain_url, NULL);
  }
  if (!isset($session) && isset($domain_id) && ($domain_url == 'force')) {
    //called from _drd_mod_content: we know session does exist, so call again to get server-session
    return _drd_get_url_key($id, NULL, NULL);
  }
  return array($plain_url, $key, $session);
}

function drd_session($id, $domain_id = NULL, $domain_url = NULL) {
  $servers = _drd_load_servers();
  list($plain_url, $key, $session) = _drd_get_url_key($id, $domain_id, $domain_url);
  $url = 'http://'. $plain_url .'/xmlrpc.php';
  if (!_drd_xmlrpc($url, 'drd.api', DRD_API_VERSION_SERVER)) {
    $error = new stdClass();
    $error->code = DRD_ERROR_WRONG_API;
    $error->message = t('Wrong API, please update your DRD_Server module.');
    Throw new Exception(_drd_xmlrpc_error_message($error, array($url, 'Check API')));
  }
  if (isset($session)) {
    if (_drd_xmlrpc($url, 'drd.session.valid', $session)) {
      return array($url, $session);
    }
    cache_clear_all($key, 'cache_drd');
  }
  $user = $servers[$id]['user'];
  $pass = $servers[$id]['password'];
  $session = _drd_xmlrpc($url, 'drd.connect', $user, $pass);
  if (!isset($session) || empty($session) || ($session === FALSE)) {
    Throw new Exception(_drd_xmlrpc_error_message(xmlrpc_error(), array($url, 'Login Failed')));
  }
  cache_set($key, array($plain_url, $session), 'cache_drd');
  return array($url, $session);
}

/**
 * Uses the first element in $areas to update content
 * Uses the last element in $areas to remove the "updating" class
 */
function _drd_mod_content($ids, $areas, $content, $extras = array()) {
  if (is_array($ids)) {
    $id = $ids[0];
    $domain_id = $ids[1];
    $selector = '#drd-dashboard #server-domain-details #id'. $id .'.server .server-domains #id'. $domain_id .' .domain-';
    $main_selector = '#drd-dashboard #server-domain-details #id'. $id .'.server .server-domains #id'. $domain_id .' .domain-';
    $time_selector_1 = '#drd-dashboard #server-domain-details #id'. $id .'.server .server-domains #id'. $domain_id .' > .last-updated';
    $time_selector_2 = '#drd-dashboard #server-domain-details #id'. $id .'.server .server-domains #id'. $domain_id .' .domain-';
  }
  else {
    $id = $ids;
    $domain_id = NULL;
    $selector = '#drd-dashboard #server-details #id'. $id .'.server .server-';
    $main_selector = '#drd-dashboard #servers #id'. $id .'.server .server-';
    $domains_selector = '#drd-dashboard #server-domain-details #id'. $id .'.server .server-';
    $time_selector_1 = '#drd-dashboard #servers #id'. $id .'.server .last-updated';
    $time_selector_2 = '#drd-dashboard #server-details #id'. $id .'.server .server-';
  }
  list($plain_url, $key, $session) = _drd_get_url_key($id, $domain_id, 'force');
  $server_selector = '#drd-dashboard #servers #id'. $id .'.server';
  if (!is_array($areas)) {
    $areas = array($areas);
  }
  if (!is_array($content)) {
    $content = array($content);
  }
  $result = array();
  foreach ($areas as $area) {
    if (substr($area, 0, 1) == '+') {
      $area = substr($area, 1);
      $cmd = 'prependContent';
    }
    else {
      $cmd = 'modContent';
    }
    $sel = ($area == 'domains') ? $domains_selector : $selector;
    $sel = (in_array($area, array('name', 'url', 'alert', 'svn-revision'))) ? $main_selector : $sel;
    if (sizeof($content)) {
      $cnt = array_shift($content);
      $cnt = str_replace('href="/', 'href="http://'. $plain_url .'/', $cnt);
      $cnt = str_replace('href="http://', 'target="_blank" href="http://', $cnt);
      $result[] = array(
        'cmd' => $cmd,
        'selector' => $sel . $area,
        'content' => $cnt,
      );
    }
  }
  $datetime = format_date(time(), 'short');
  $first_area = array_shift($areas);
  if (substr($first_area, 0, 1) == '+') {
    $first_area = substr($first_area, 1);
  }
  $result[] = array(
    'cmd' => 'removeClass',
    'selector' => $main_selector .'toolbar .'. $area,
    'value' => 'updating',
  );
  $result[] = array(
    'cmd' => 'removeClass',
    'selector' => $main_selector .'info-box .'. $area,
    'value' => 'updating',
  );
  $result[] = array(
    'cmd' => 'modContent',
    'selector' => $time_selector_2 .'info-box .'. $area .' ~ .last-updated',
    'content' => $datetime,
  );
  $result[] = array(
    'cmd' => 'modContent',
    'selector' => $time_selector_1,
    'content' => $datetime,
  );
  foreach ($extras as $key => $extra) {
    if (isset($extra['selector'])) {
      $extras[$key]['selector'] = str_replace('[selector]', $selector, $extras[$key]['selector']);
      $extras[$key]['selector'] = str_replace('[server-selector]', $server_selector, $extras[$key]['selector']);
    }
  }
  $result = array_merge($result, $extras);
  $result[] = array(
    'cmd' => 'updateSelected',
    'selector' => $selector . $first_area,
  );
  return $result;
}

function drd_dashboard_server_info($id) {
  list($url, $session) = drd_session($id);
  if (!isset($url)) {
    return _drd_mod_content($id, array('+message-list .content', 'info'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.info', $session);
  $output = theme('drd_domain_info', array('items' => $info['items']));
  return _drd_mod_content($id, array('name', 'info .content', 'info'), array($info['site_name'], $output));
}

function drd_dashboard_server_domains($id) {
  list($url, $session) = drd_session($id);
  if (!isset($url)) {
    return _drd_mod_content($id, 'domains', $session->message);
  }
  // Get all Cache data for all the domains so that we can restore those values
  // for each domain that still exsts
  $old_domains = array();
  $did = 1;
  while ($cache = cache_get(drd_cache_id($id, $did, 'DomainInfo'), 'cache_drd')) {
    $cache2 = cache_get(drd_cache_id($id, $did, 'DomainListUpdates'), 'cache_drd');
    $old_domains[$cache->data['url']] = array(
      'DomainInfo' => $cache->data['value'],
      'DomainListUpdates' => $cache2->data['value'],
    );
    $did++;
  }
  cache_clear_all(drd_cache_id($id), 'cache_drd', TRUE);

  $domains = _drd_xmlrpc($url, 'drd.server.domains', $session);
  $output = '';
  $servers = _drd_load_servers();
  $domain_id = 0;
  foreach ($domains as $domain_dir => $domain) {
    $domain_id++;
    if (strstr($servers[$id]['disabled_domains'], $domain['url'])) {
      continue;
    }
    if (array_key_exists($domain['url'], $old_domains)) {
      foreach ($old_domains[$domain['url']] as $op => $value) {
        $cid = drd_cache_id($id, $domain_id, $op);
        cache_set($cid, array('url' => $domain['url'], 'value' => $value), 'cache_drd');
      }
    }
    $output .= '<div id="id'. $domain_id .'" class="domain clearfix">';
    $output .= '  <div class="domain-dir">'. $domain_dir .'</div>';
    $output .= '  <div class="domain-toggle ui-icon ui-icon-triangle-1-s">v</div>';
    $output .= '  <div class="toolbar domain-toolbar">'. _drd_domain_toolbar() .'</div>';
    $output .= '  <div class="domain-name">'. t('Unknown') .'</div>';
    $output .= '  <div class="last-updated">&nbsp;</div>';
    $output .= _drd_icon_alerts('domain');
    $output .= '  <div class="domain-url">';
    if (sizeof($domain['aliases'])) {
      $output .= '  <div class="domain-alias-toggle ui-icon ui-icon-triangle-1-s">v</div>';
      $output .= '  <div class="domain-aliases element-hidden">';
      $output .= '    <div class="title">'. t('Other URLs (aliases)') .'</div>';
      foreach ($domain['aliases'] as $alias) {
        $output .= '  <div class="domain-alias">'. $alias .'</div>';
      }
      $output .= '  </div>';
    }
    $output .= '    <div class="plain-url element-hidden">'. $domain['url'] .'</div>';
    $output .= '    <div class="url">'. l($domain['url'], 'http://'. $domain['url'], array('external' => TRUE, 'attributes' => array('target' => '_blank'))) .'</div>';
    $output .= '  </div>';
    $output .= '  <div class="domain-details element-hidden">';
    $output .= _drd_info_box('domain', 'message-list', t('Messages'));
    $output .= _drd_info_box('domain', 'info', t('Status Report'));
    $output .= _drd_info_box('domain', 'list-updates', t('Available Updates'));
    $output .= '  </div>';
    $output .= '</div>';
  }
  return _drd_mod_content($id, 'domains', $output);
}

function drd_dashboard_server_svn_status($id) {
  list($url, $session) = drd_session($id);
  if (!isset($url)) {
    return _drd_mod_content($id, array('svn-status .content', 'svn-status'), $session->message);
  }
  $servers = _drd_load_servers();
  $svn_user = $servers[$id]['svn']['user'];
  $svn_password = $servers[$id]['svn']['password'];
  $force_external = $servers[$id]['svn']['force_external'];
  $svn_status = _drd_xmlrpc($url, 'drd.server.svn.status', $session, $svn_user, $svn_password, $force_external);
  $info = array_pop($svn_status[1]);
  $output = theme('drd_svn_status', array('items' => $svn_status[1])) .'<div class="info">'. $info['info'] .'</div>';
  return _drd_mod_content($id, array('svn-revision', 'svn-status .content', 'svn-status'), array($svn_status[0], $output));
}

function drd_dashboard_server_svn_update($id) {
  list($url, $session) = drd_session($id);
  if (!isset($url)) {
    return _drd_mod_content($id, array('svn-update .content', 'svn-update'), $session->message);
  }
  $servers = _drd_load_servers();
  $svn_user = $servers[$id]['svn']['user'];
  $svn_password = $servers[$id]['svn']['password'];
  $force_external = $servers[$id]['svn']['force_external'];
  $svn_update = _drd_xmlrpc($url, 'drd.server.svn.update', $session, $svn_user, $svn_password, $force_external);
  $info = array_pop($svn_update[1]);
  $output = theme('drd_svn_update', array('items' => $svn_update[1])) .'<div class="info">'. $info['info'] .'</div>';
  return _drd_mod_content($id, array('svn-revision', 'svn-update .content', 'svn-update'), array($svn_update[0], $output), _drd_get_extra_remove_svn_status());
}

function drd_dashboard_server_php_error_log($id) {
  list($url, $session) = drd_session($id);
  if (!isset($url)) {
    return _drd_mod_content($id, array('php-error-log .content', 'php-error-log'), $session->message);
  }
  $log = _drd_xmlrpc($url, 'drd.server.php.error.log', $session);
  $log = str_replace("\r", "", $log[0]);
  $log = str_replace("\n\n", "\n", $log);
  $log = explode("\n", $log);
  $log = array_reverse($log);
  $output = '<pre>'. implode("\n", $log) .'</pre>';
  return _drd_mod_content($id, array('php-error-log .content', 'php-error-log'), $output);
}

function drd_dashboard_domain_info($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'info'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.info', $session, TRUE);
  $drd_warning = $info['system_status_warning'];
  $drd_error = $info['system_status_error'];
  $output = theme('drd_domain_info', array('items' => $info['items']));
  $extras = array();
  $warning_cmd = ($drd_warning) ? 'show' : 'hide';
  $error_cmd = ($drd_error) ? 'show' : 'hide';
  $extras[] = array(
    'cmd' => $warning_cmd,
    'selector' => '[selector]alert-warning .toggle-alert',
  );
  $extras[] = array(
    'cmd' => 'setData',
    'selector' => '[selector]alert-warning .toggle-alert',
    'name' => 'drdWarning',
    'value' => $drd_warning,
  );
  $extras[] = array(
    'cmd' => $error_cmd,
    'selector' => '[selector]alert-error .toggle-alert',
  );
  $extras[] = array(
    'cmd' => 'setData',
    'selector' => '[selector]alert-error .toggle-alert',
    'name' => 'drdError',
    'value' => $drd_error,
  );
  $extras[] = array(
    'cmd' => 'checkServerAlert',
    'selector' => '[server-selector]',
  );
  return _drd_mod_content(array($id, $domain_id), array('name', 'info .content', 'info'), array($info['site_name'], $output), $extras);
}

function drd_dashboard_domain_flush_cache($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'flush-cache'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.cache.flush', $session, TRUE);
  return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'flush-cache'), $info);
}

function drd_dashboard_domain_run_cron($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'run-cron'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.run.cron', $session, TRUE);
  return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'run-cron'), $info, _drd_get_extra_remove_domain_info());
}

function drd_dashboard_domain_list_updates($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'list-updates'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.list.updates', $session, TRUE);
  $output  = '<div class="info-box availables-updates">';
  $output .= $info['upd'];
  $output .= '</div>';
  return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'list-updates .content', 'list-updates'), array($info['msg'], $output), _drd_get_extra_remove_domain_info());
}

function drd_dashboard_domain_run_update($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'run-update'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.run.update', $session, TRUE);
  return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'run-update'), $info, _drd_get_extra_remove_domain_info());
}

function drd_dashboard_domain_switch_maintenance($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'switch-maintenance'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.switch.maintenance', $session, TRUE);
  return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'switch-maintenance'), $info, _drd_get_extra_remove_domain_info());
}

function drd_dashboard_domain_update_translation($id, $domain_id, $domain_url) {
  list($url, $session) = drd_session($id, $domain_id, $domain_url);
  if (!isset($url)) {
    return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'update-translation'), $session->message);
  }
  $info = _drd_xmlrpc($url, 'drd.update.translation', $session, TRUE);
  return _drd_mod_content(array($id, $domain_id), array('+message-list .content', 'update-translation'), $info, _drd_get_extra_remove_domain_info());
}

function _drd_get_extra_remove_domain_info() {
  return array(
    array(
      'cmd' => 'addClass',
      'selector' => '[selector]toolbar .info',
      'value' => 'drd-run-once',
    ),
  );
}

function _drd_get_extra_remove_svn_status() {
  return array(
    array(
      'cmd' => 'addClass',
      'selector' => '[server-selector] .toolbar .svn-status',
      'value' => 'drd-run-once',
    ),
  );
}

function drd_dashboard_edit_server_form($form, $form_state) {
  $form = array();
  $form['disabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable the server'),
  );
  $form['weight'] = array(
    '#type' => 'textfield',
    '#title' => t('Weight'),
    '#size' => 5,
  );
  $form['url'] = array(
    '#type' => 'textfield',
    '#title' => t('URL of the server'),
    '#size' => 30,
  );
  $form['user'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('This user needs to have permission to administer the remote Drupal site.'),
    '#size' => 30,
  );
  $form['password'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#size' => 30,
  );
  $form['svn'] = array(
    '#type' => 'fieldset',
    '#title' => t('SVN Settings'),
  );
  $form['svn']['svn_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable SVN'),
  );
  $form['svn']['svn_user'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#description' => t('The user for the corresponding SVN repository.'),
    '#size' => 25,
  );
  $form['svn']['svn_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Password'),
    '#size' => 25,
  );
  $form['svn']['svn_force_external'] = array(
    '#type' => 'checkbox',
    '#title' => t('Force commandline SVN'),
  );
  $form['domains'] = array(
    '#type' => 'fieldset',
    '#title' => t('Domains'),
  );
  $form['domains']['disabled_domains'] = array(
    '#type' => 'textarea',
    '#title' => t('Disabled Domains'),
    '#rows' => 3,
    '#resizable' => FALSE,
  );
  $form['domains']['all_domains'] = array(
    '#markup' => '<div class="all-domains">&nbsp;</div>',
  );
  $form['id'] = array(
    '#type' => 'hidden',
    '#value' => -1,
  );
  $form['buttons'] = array(
    '#type' => 'fieldset',
  );
  $form['buttons']['save'] = array(
    '#markup' => '<div class="button save-button">'. t('Save') .'</div>',
  );
  $form['buttons']['cancel'] = array(
    '#markup' => '<div class="button cancel-button">'. t('Cancel') .'</div>',
  );
  return $form;
}

function drd_dashboard_edit_server($id) {
  $servers = _drd_load_servers();
  $edit = isset($servers[$id]);
  return array(
    array(
      'cmd' => 'EditServer',
      'id' => $id,
      'weight' => ($edit) ? $servers[$id]['weight'] : 0,
      'disabled' => ($edit) ? $servers[$id]['disabled'] : FALSE,
      'url' => ($edit) ? $servers[$id]['url'] : '',
      'user' => ($edit) ? $servers[$id]['user'] : '',
      'password' => ($edit) ? $servers[$id]['password'] : '',
      'svn_enabled' => ($edit) ? $servers[$id]['svn']['enabled'] : FALSE,
      'svn_user' => ($edit) ? $servers[$id]['svn']['user'] : '',
      'svn_password' => ($edit) ? $servers[$id]['svn']['password'] : '',
      'svn_force_external' => ($edit) ? $servers[$id]['svn']['force_external'] : TRUE,
      'disabled_domains' => ($edit) ? $servers[$id]['disabled_domains'] : '',
    ),
    array(
      'cmd' => 'removeClass',
      'selector' => '#drd-dashboard #servers #id'. $id .'.server .server-toolbar .edit',
      'value' => 'updating',
    ),
    array(
      'cmd' => 'removeClass',
      'selector' => '#drd-dashboard #server-toolbars .general-toolbar .new',
      'value' => 'updating',
    ),
  );
}

function drd_dashboard_save_server($id) {
  $servers = _drd_load_servers();
  $server = array(
    'weight' => $_POST['weight'],
    'disabled' => ($_POST['disabled'] == 'true'),
    'url' => $_POST['url'],
    'user' => $_POST['user'],
    'password' => $_POST['password'],
    'svn' => array(
      'enabled' => ($_POST['svn_enabled'] == 'true'),
      'user' => $_POST['svn_user'],
      'password' => $_POST['svn_password'],
      'force_external' => ($_POST['svn_force_external'] == 'true'),
    ),
    'disabled_domains' => $_POST['disabled_domains'],
  );
  if ($id < 0) {
    $id = sizeof($servers);
  }
  $servers[$id] = $server;
  _drd_save_servers($servers);
  list($part_main, $part_domain, $part_server) = _drd_get_server_markup($id, $servers[$id]['url'], $servers[$id]['disabled'], $servers[$id]['svn']['enabled']);
  $result = drd_dashboard_delete_server($id, FALSE);
  $result[0]['cmd'] = 'replaceContent';
  $result[0]['content'] = $part_main;
  $result[] = array(
    'cmd' => 'appendContent',
    'selector' => '#drd-dashboard #server-details',
    'content' => $part_server,
  );
  $result[] = array(
    'cmd' => 'appendContent',
    'selector' => '#drd-dashboard #server-domain-details',
    'content' => $part_domain,
  );
  $result[] = array(
    'cmd' => 'reload',
  );
  return $result;
}

function drd_dashboard_delete_server($id, $mark_deleted = TRUE) {
  if ($mark_deleted) {
    $servers = _drd_load_servers();
    $servers[$id]['deleted'] = TRUE;
    _drd_save_servers($servers);
  }
  $result = array();
  $result[] = array(
    'cmd' => 'removeContent',
    'selector' => '#drd-dashboard #servers #id'.$id,
  );
  $result[] = array(
    'cmd' => 'removeContent',
    'selector' => '#drd-dashboard #server-details #id'.$id,
  );
  $result[] = array(
    'cmd' => 'removeContent',
    'selector' => '#drd-dashboard #server-domain-details #id'.$id,
  );
  return $result;
}

function drd_dashboard_flush_cache() {
  cache_clear_all('*', 'cache_drd', TRUE);
  $result = array();
  $result[] = array(
    'cmd' => 'alert',
    'content' => 'DRD Cache has been flushed, please reload this page.',
  );
  return $result;
}

function _drd_xmlrpc($url) {
  require_once DRUPAL_ROOT . '/includes/xmlrpc.inc';
  $args = func_get_args();
  $url = array_shift($args);
  $func = array_shift($args);
  $func_args = array(
    $func => $args,
  );
  $options = array('timeout' => 300,);
  $xmlrpc_args = array($url, $func_args, $options);
  $output = call_user_func_array('_xmlrpc', $xmlrpc_args);
  if ($output === FALSE) {
    Throw new Exception(_drd_xmlrpc_error_message(xmlrpc_error(), $xmlrpc_args));
  }
  $output = drupal_json_decode($output);
  if ($output['is_error'] === TRUE || $output['is_error'] === '1' || $output['is_error'] === 'TRUE' || $output['is_error'] === 'true') {
    $error = new stdClass();
    $error->code = $output['code'];
    $error->message = $output['message'];
    Throw new Exception(_drd_xmlrpc_error_message($error, $xmlrpc_args));
  }
  return $output['message'];
}

function _drd_xmlrpc_error_message($error, $args = array()) {
  if (!isset($error) || !isset($error->message)) {
    $error = new stdClass();
    $error->code = -1;
    $error->message = 'Unknown error';
  }
  return '['. $error->code .'] '. $error->message .' ('. $args[0] .'->'. $args[1] .')';
}
